dir="$(dirname $0)"
cd "$dir"

# Pre-cleanup
rm -rf cov/ hpc/

# Make per-executable directories
for name in stack-test-exe-1 stack-test-exe-2 lib
do
  mkdir -p hpc/$name
done

# Rewrite the cabal file to point to the per-executable hpcdir
if [ -f stack-test.cabal.orig ]
then
  echo "There's a stack-test.cabal.orig , which means the last run didn't cleanup; aborting."
  exit 1
else
  mv stack-test.cabal stack-test.cabal.orig
fi
sed -e '/^\s*ghc-options:/d' -e 's/-- cov: //g' stack-test.cabal.orig >stack-test.cabal

# Force a rebuild ; --force-dirty doesn't (as of 2017-08-07) do the
# right thing; see
# https://github.com/commercialhaskell/stack/issues/3306 and
# https://github.com/commercialhaskell/stack/issues/1940
for file in $(find app/ src/ -type f)
do
  sed -i '$s/$/ /' $file
done

stack build stack-test
stack install stack-test

# Here we run the tests; normally this would be "stack test" but for
# this example I couldn't be bothered in setting up an executable
# shim.
~/.local/bin/stack-test-exe-1
~/.local/bin/stack-test-exe-2

# Here we find the .tix files, sum them just in case there's more
# than one (as when you have more than one test against the same
# executable but run from different directories), put them where we
# want them, and clean up the originals.
#
# It's arguable that this step is stack's job, but if stack isn't
# going to do it but stack is going to generate the final markup,
# then there probably has to be some sort of hook to run a user
# script to do it, as "I need to run the executable in the test
# directory" is not an unreasonable requirement.
for name in stack-test-exe-1 stack-test-exe-2
do
  mkdir -p cov/$name
  tixes=$(find . -name hpc -prune -o -type f -name $name.tix -print | tr '\n' ' ')
  if [ "$tixes" ]
  then
    rm -f hpc/$name.tix

    # The hpc directory for each executable needs to be able to see
    # the hpc directory for the library, which has a weird name like
    # stack-test-0.1.0.0-3m1Ipo3mDvwHgd4ACToob3 ; if not we get an
    # error like so:
    #
    #   stack exec hpc -- markup --hpcdir=hpc/stack-test-exe-1/ --destdir=cov/stack-test-exe-1/ --verbosity=2 hpc/stack-test-exe-1.tix
    #   hpc: can not find stack-test-0.1.0.0-3m1Ipo3mDvwHgd4ACToob3/Lib in ./.hpc, ./hpc/stack-test-exe-1/
    #   CallStack (from HasCallStack):
    #     error, called at libraries/hpc/Trace/Hpc/Mix.hs:122:15 in hpc-0.6.0.3:Trace.Hpc.Mix
    #
    # We solve this with a symlink:
    ln -s ../lib/$(cd hpc/lib ; \ls)  hpc/$name/

    stack exec hpc sum $tixes >hpc/$name.tix

    /bin/rm -f ./.stack-work/dist/x86_64-linux/*/hpc/*
    /bin/rm -f ./.hpc/*
    /bin/rm -f $tixes
  fi
done

# Here we generate the markup.
for name in stack-test-exe-1 stack-test-exe-2
do
  if [ -f hpc/$name.tix ]
  then
    echo stack exec hpc -- markup --hpcdir=hpc/$name/ --destdir=cov/$name/ --verbosity=2 hpc/$name.tix
    stack exec hpc -- markup --hpcdir=hpc/$name/ --destdir=cov/$name/ --verbosity=2 hpc/$name.tix
  fi
done

# Cleanup
if [ -f stack-test.cabal.orig ]
then
  mv stack-test.cabal.orig stack-test.cabal
fi

# Undo what we did to force a rebuild
for file in $(find app/ src/ -type f)
do
  sed -i '$s/ *$//' $file
done
